<?php

/** Anleitung:
Dieses Script als Datei "check_civicrm_requirements.php" in einem über den Browser aufrufbaren Verzeichnis abspeichern und im Browser
aurufen. Z.B. https://meinedomain.xy/check_civicrm_requirements.php 
Das Script liest bzw. sucht nach bestimmten Systemparametern und gibt abhängig von den Befunden Meldungen aus.*/

header('Content-Type: text/plain');

function check_php_version() {
    return version_compare(PHP_VERSION, '7.3', '>=')
        ? "✔ PHP-Version: " . PHP_VERSION
        : "✘ PHP-Version zu niedrig: " . PHP_VERSION;
}

function check_php_extensions() {
    $required_extensions = ['curl', 'dom', 'gd', 'json', 'mbstring', 'mysqli', 'SimpleXML', 'xml', 'zip'];
    $missing = array_filter($required_extensions, fn($ext) => !extension_loaded($ext));
    return empty($missing)
        ? "✔ Alle erforderlichen PHP-Erweiterungen sind installiert."
        : "✘ Fehlende PHP-Erweiterungen: " . implode(', ', $missing);
}

function check_php_settings() {
    $settings = [
        'memory_limit' => 256,
        'max_execution_time' => 240,
        'max_input_time' => 120,
        'post_max_size' => 50,
        'upload_max_filesize' => 50
    ];
    
    $issues = [];
    foreach ($settings as $setting => $recommended) {
        $current = ini_get($setting);
        $current_value = convert_to_megabytes($current);
        
        if ($current_value < $recommended) {
            $issues[] = "✘ $setting zu niedrig ($current, empfohlen: {$recommended}M)";
        } else {
            $issues[] = "✔ $setting: $current";
        }
    }
    
    return implode("\n", $issues);
}

function convert_to_megabytes($value) {
    $unit = strtolower(substr($value, -1));
    $bytes = (int) $value;

    switch($unit) {
        case 'g':
            $bytes *= 1024;
            break;
        case 'k':
            $bytes /= 1024;
            break;
    }

    return $bytes;
}

function check_database() {
    ob_start();
    passthru('mysql --version 2>&1');
    $output = ob_get_clean();
    if (strpos($output, 'MySQL') !== false || strpos($output, 'MariaDB') !== false) {
        return "✔ Datenbank erkannt: " . trim($output);
    }
    return "✘ Keine unterstützte Datenbank gefunden (MySQL oder MariaDB erforderlich).";
}

function check_webserver() {
    if (isset($_SERVER['SERVER_SOFTWARE'])) {
        return "✔ Webserver: " . $_SERVER['SERVER_SOFTWARE'];
    }
    return "✘ Kein unterstützter Webserver erkannt.";
}

$checks = [
    check_php_version(),
    check_php_extensions(),
    check_php_settings(),
    check_database(),
    check_webserver()
];

echo "Einführung\n";
echo "--------------------------------\n";
echo "Dieses Script ist am 16.02.2025 im Dialog mit ChatGPT.com entstanden mit dem Ziel einer einfachen Möglichkeit erkennen zu können,
ob eine Installation von CiviCRM (https://civicrm.org) in einer bestimmten Laufzeitumgebung möglich ist. Zum Beispiel, wenn man
ein Webhosting-Paket bei einem Webhosting-Anbieter wie netcup.de, alfahosting.de, strato.de oder einem der vielen anderen
Anbieter gebucht hat und dort eine Instanz von CiviCRM installieren und betreiben möchte, so wie man dort vielleicht auch bereits
ein WordPress-Weblog oder dergleichen installiert hat und betreibt.\n
Das Script basiert auf den Anforderungen CiviCRM-Version 5.82.0\n
Unter Umständen muss dieses Script nochmals überprüft und aktualisiert werden, falls sich die Anforderungen bei Folgeversionen von CiviCRM verändern.\n
Auch ist es interessant zu wissen, dass die Installation auch dann gelingen kann, wenn das memory_limit oder die max_execution_time geringer ausfallen
als die offiziell verlangten Werte. Diese Werte sind nämlich ggf. eher für operative Funktionen des Systems relevant, als für den Installationsprozess und es 
kann sein, dass man diejenigen Funktionen von CiviCRM, die diese Werte gemäß der Erfahrungen der Entwickler benötigen, gar nicht benutzt oder die verarbeiteten Datenmengen
ausreichend klein sind, um auch mit geringeren Werten zurecht zu kommen.\n
Positive Erfahrungswerte mit der Installation von CiviCRM bei netcup.de gibt es bereits bei einer max_execution_time  von 180 (empfohlen: 240)
und einer max_input_time von 60, empfohlen: 120M)\n";
echo "\n";
echo "CiviCRM Systemanforderungen Check\n";
echo "--------------------------------\n";
echo implode("\n", $checks);

echo "\n\nZusammenfassung:";
$failed = array_filter($checks, fn($result) => str_starts_with($result, '✘'));
if (empty($failed)) {
    echo "\n✅ Das System erfüllt alle Anforderungen für CiviCRM.";
} else {
    echo "\n❌ Das System erfüllt nicht alle Anforderungen für CiviCRM.";
}

echo "\n\nSollten einzelne Variablen nicht ausreichen, sollte geprüft werden, ob man nicht ggf. über die Verwaltungsoberfläche des Webhosting-Pakets, 
oder, falls man sogar über root-Rechte verfügt, durch die Veränderungen von Systemeinstellungen entsprechende Bedingungen einstellen kann.\n
Dies sollte im einfachsten Fall meist sowohl für die PHP-Version als auch für PHP-Variablen möglich sein.";